searchData={"items":[{"type":"module","title":"Mimic","doc":"Mimic is a library that simplifies the usage of mocks in Elixir.\n\nMimic is mostly API compatible with [mox](https://hex.pm/packages/mox) but\ndoesn't require explicit contract checking with behaviours.  It's also faster.\nYou're welcome.\n\nMimic works by copying your module out of the way and replacing it with one of\nit's own which can delegate calls back to the original or to a mock function\nas required.\n\nIn order to prepare a module for mocking you must call `copy/1` with the\nmodule as an argument.  We suggest that you do this in your\n`test/test_helper.exs`:\n\n```elixir\nMimic.copy(Calculator)\nExUnit.start()\n```\n\nImportantly calling `copy/1` will not change the behaviour of the module. When\nwriting tests you can then use `stub/3` or `expect/3` to add mocks and\nassertions.","ref":"Mimic.html"},{"type":"module","title":"Multi-process collaboration - Mimic","doc":"Mimic supports multi-process collaboration via two mechanisms:\n\n  1. Explicit allows.\n  2. Global mode.\n\nUsing explicit allows is generally preferred as these stubs can be run\nconcurrently, whereas global mode tests must be run exclusively.","ref":"Mimic.html#module-multi-process-collaboration"},{"type":"module","title":"Explicit allows - Mimic","doc":"Using `allow/3` you can give other processes permission to use stubs and\nexpectations from where they were not defined.\n\n```elixir\ntest \"invokes add from a process\" do\n  Calculator\n  |> expect(:add, fn x, y -> x + y end)\n\n  parent_pid = self()\n\n  spawn_link(fn ->\n    Calculator |> allow(parent_pid, self())\n    assert Calculator.add(2, 3) == 5\n\n    send parent_pid, :ok\n  end)\n\n  assert_receive :ok\nend\n```\n\nIf you are using `Task` the expectations and stubs are automatically allowed","ref":"Mimic.html#module-explicit-allows"},{"type":"module","title":"Global mode - Mimic","doc":"When set in global mode any process is able to call the stubs and expectations\ndefined in your tests.\n\n**Warning: If using global mode you should remove `async: true` from your tests**\n\nEnable global mode using `set_mimic_global/1`.\n\n```elixir\nsetup :set_mimic_global\nsetup :verify_on_exit!\n\ntest \"invokes add from a task\" do\n  Calculator\n  |> expect(:add, fn x, y -> x + y end)\n\n  Task.async(fn ->\n    assert Calculator.add(2, 3) == 5\n  end)\n  |> Task.await\nend\n```","ref":"Mimic.html#module-global-mode"},{"type":"function","title":"Mimic.allow/3","doc":"Allow other processes to share expectations and stubs defined by another\nprocess.","ref":"Mimic.html#allow/3"},{"type":"function","title":"Arguments: - Mimic.allow/3","doc":"* `module` - the copied module.\n  * `owner_pid` - the process ID of the process which created the stub.\n  * `allowed_pid` - the process ID of the process which should also be allowed\n    to use this stub.","ref":"Mimic.html#allow/3-arguments"},{"type":"function","title":"Raises: - Mimic.allow/3","doc":"* If Mimic is running in global mode.\n\nAllows other processes to share expectations and stubs defined by another\nprocess.","ref":"Mimic.html#allow/3-raises"},{"type":"function","title":"Example - Mimic.allow/3","doc":"```elixir\ntest \"invokes add from a task\" do\n  Calculator\n  |> expect(:add, fn x, y -> x + y end)\n\n  parent_pid = self()\n\n  Task.async(fn ->\n    Calculator |> allow(parent_pid, self())\n    assert Calculator.add(2, 3) == 5\n  end)\n  |> Task.await\nend\n```","ref":"Mimic.html#allow/3-example"},{"type":"function","title":"Mimic.copy/1","doc":"Prepare `module` for mocking.\n\nIdeally, don't call this function twice for the same module, but in case you do, this function\nis idempotent. It will not delete any `stub`s or `expect`s that you've set up.","ref":"Mimic.html#copy/1"},{"type":"function","title":"Arguments: - Mimic.copy/1","doc":"* `module` - the name of the module to copy.","ref":"Mimic.html#copy/1-arguments"},{"type":"function","title":"Mimic.expect/4","doc":"Define a stub which must be called within an example.\n\nThis function is almost identical to `stub/3` except that the replacement\nfunction must be called within the lifetime of the calling `pid` (i.e. the\ntest example).","ref":"Mimic.html#expect/4"},{"type":"function","title":"Arguments: - Mimic.expect/4","doc":"* `module` - the name of the module in which we're adding the stub.\n  * `function_name` - the name of the function we're stubbing.\n  * `function` - the function to use as a replacement.","ref":"Mimic.html#expect/4-arguments"},{"type":"function","title":"Raises: - Mimic.expect/4","doc":"* If `module` is not copied.\n  * If `function_name` is not publicly exported from `module` with the same\n    arity.\n  * If `function` is not called by the stubbing process.","ref":"Mimic.html#expect/4-raises"},{"type":"function","title":"Example - Mimic.expect/4","doc":"iex> Calculator.add(2, 4)\n    6\n\n    iex> Mimic.expect(Calculator, :add, fn x, y -> x * y end)\n    ...> Calculator.add(2, 4)\n    8","ref":"Mimic.html#expect/4-example"},{"type":"function","title":"Mimic.mode/0","doc":"Returns the current mode (`:global` or `:private`)","ref":"Mimic.html#mode/0"},{"type":"function","title":"Mimic.reject/1","doc":"Define a stub which must not be called.\n\nThis function allows you do define a stub which must not be called during the\ncourse of this test.  If it is called then the verification step will raise.","ref":"Mimic.html#reject/1"},{"type":"function","title":"Arguments: - Mimic.reject/1","doc":"* `function` - A capture of the function which must not be called.","ref":"Mimic.html#reject/1-arguments"},{"type":"function","title":"Raises: - Mimic.reject/1","doc":"* If `function` is not called by the stubbing process while calling `verify!/1`.","ref":"Mimic.html#reject/1-raises"},{"type":"function","title":"Example: - Mimic.reject/1","doc":"iex> Mimic.reject(&Calculator.add/2)\n    Calculator","ref":"Mimic.html#reject/1-example"},{"type":"function","title":"Mimic.reject/3","doc":"Define a stub which must not be called.\n\nThis function allows you do define a stub which must not be called during the\ncourse of this test.  If it is called then the verification step will raise.","ref":"Mimic.html#reject/3"},{"type":"function","title":"Arguments: - Mimic.reject/3","doc":"* `module` - the name of the module in which we're adding the stub.\n  * `function_name` - the name of the function we're stubbing.\n  * `arity` - the arity of the function we're stubbing.","ref":"Mimic.html#reject/3-arguments"},{"type":"function","title":"Raises: - Mimic.reject/3","doc":"* If `function` is not called by the stubbing process while calling `verify!/1`.","ref":"Mimic.html#reject/3-raises"},{"type":"function","title":"Example: - Mimic.reject/3","doc":"iex> Mimic.reject(Calculator, :add, 2)\n    Calculator","ref":"Mimic.html#reject/3-example"},{"type":"function","title":"Mimic.set_mimic_from_context/1","doc":"Chooses the mode based on ExUnit context. If `async` is `true` then\nthe mode is private, otherwise global.\n\n```elixir\nsetup :set_mimic_from_context\n```","ref":"Mimic.html#set_mimic_from_context/1"},{"type":"function","title":"Mimic.set_mimic_global/1","doc":"Sets the mode to global. Mocks can be set and used by all processes\n\n```elixir\nsetup :set_mimic_global\n```","ref":"Mimic.html#set_mimic_global/1"},{"type":"function","title":"Mimic.set_mimic_private/1","doc":"Sets the mode to private. Mocks can be set and used by the process\n\n```elixir\nsetup :set_mimic_private\n```","ref":"Mimic.html#set_mimic_private/1"},{"type":"function","title":"Mimic.stub/1","doc":"Replace all public functions in `module` with stubs.\n\nThe stubbed functions will raise if they are called.","ref":"Mimic.html#stub/1"},{"type":"function","title":"Arguments: - Mimic.stub/1","doc":"* `module` - The name of the module to stub.","ref":"Mimic.html#stub/1-arguments"},{"type":"function","title":"Raises: - Mimic.stub/1","doc":"* If `module` is not copied.\n  * If `function` is not called by the stubbing process.","ref":"Mimic.html#stub/1-raises"},{"type":"function","title":"Example - Mimic.stub/1","doc":"iex> Mimic.stub(Calculator)\n    ...> Calculator.add(2, 4)\n    ** (ArgumentError) Module Calculator has not been copied.  See docs for Mimic.copy/1","ref":"Mimic.html#stub/1-example"},{"type":"function","title":"Mimic.stub/3","doc":"Define a stub function for a copied module.","ref":"Mimic.html#stub/3"},{"type":"function","title":"Arguments: - Mimic.stub/3","doc":"* `module` - the name of the module in which we're adding the stub.\n  * `function_name` - the name of the function we're stubbing.\n  * `function` - the function to use as a replacement.","ref":"Mimic.html#stub/3-arguments"},{"type":"function","title":"Raises: - Mimic.stub/3","doc":"* If `module` is not copied.\n  * If `function_name` is not publicly exported from `module` with the same arity.","ref":"Mimic.html#stub/3-raises"},{"type":"function","title":"Example - Mimic.stub/3","doc":"iex> Calculator.add(2, 4)\n    6\n\n    iex> Mimic.stub(Calculator, :add, fn x, y -> x * y end)\n    ...> Calculator.add(2, 4)\n    8","ref":"Mimic.html#stub/3-example"},{"type":"function","title":"Mimic.stub_with/2","doc":"Replace all public functions in `module` with public function in `mocking_module`.\n\nIf there's any public function in `module` that are not in `mocking_module`, it will raise if it's called","ref":"Mimic.html#stub_with/2"},{"type":"function","title":"Arguments: - Mimic.stub_with/2","doc":"* `module` - The name of the module to stub.\n  * `mocking_module` - The name of the mocking module to stub the original module.","ref":"Mimic.html#stub_with/2-arguments"},{"type":"function","title":"Raises: - Mimic.stub_with/2","doc":"* If `module` is not copied.\n  * If `function` is not called by the stubbing process.","ref":"Mimic.html#stub_with/2-raises"},{"type":"function","title":"Example - Mimic.stub_with/2","doc":"defmodule InverseCalculator do\n      def add(a, b), do: a - b\n    end\n\n    iex> Mimic.stub_with(Calculator, InverseCalculator)\n    ...> Calculator.add(2, 4)\n    ...> -2","ref":"Mimic.html#stub_with/2-example"},{"type":"function","title":"Mimic.verify!/1","doc":"Verify if expectations were fulfilled for a process `pid`","ref":"Mimic.html#verify!/1"},{"type":"function","title":"Mimic.verify_on_exit!/1","doc":"Verifies the current process after it exits.\n\nIf you want to verify expectations for all tests, you can use\n`verify_on_exit!/1` as a setup callback:\n\n```elixir\nsetup :verify_on_exit!\n```","ref":"Mimic.html#verify_on_exit!/1"},{"type":"module","title":"Mimic.Cover","doc":"Ensures mocked modules still get coverage data despite being replaced by `Mimic.Module`.\nAlso ensures that coverage data from before moving the module around is not lost","ref":"Mimic.Cover.html"},{"type":"function","title":"Mimic.Cover.enabled_for?/1","doc":"","ref":"Mimic.Cover.html#enabled_for?/1"},{"type":"module","title":"Mimic.DSL","doc":"Stubs and expectations can be expressed in a more natural way.\n\n```elixir\nuse Mimic.DSL\n```\n\n```elixir\ntest \"basic example\" do\n  stub Calculator.add(_x, _y), do: :stub\n  expect Calculator.add(x, y), do: x + y\n  expect Calculator.mult(x, y), do: x * y\n\n  assert Calculator.add(2, 3) == 5\n  assert Calculator.mult(2, 3) == 6\n\n  assert Calculator.add(2, 3) == :stub\nend\n```\n\nSupport for expecting multiple calls:\n\n```elixir\nexpect Calculator.add(x, y), num_calls: 2 do\n  x + y\nend\n```","ref":"Mimic.DSL.html"},{"type":"macro","title":"Mimic.DSL.expect/3","doc":"","ref":"Mimic.DSL.html#expect/3"},{"type":"macro","title":"Mimic.DSL.stub/2","doc":"","ref":"Mimic.DSL.html#stub/2"},{"type":"extras","title":"License","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"{}\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2012 Plataformatec\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Overview","doc":"![Mimic logo](logo.png)\n# Mimic\n\n[![CI](https://github.com/edgurgel/mimic/actions/workflows/main.yml/badge.svg)](https://github.com/edgurgel/mimic/actions/workflows/main.yml)\n[![Module Version](https://img.shields.io/hexpm/v/mimic.svg)](https://hex.pm/packages/mimic)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/mimic/)\n[![Total Download](https://img.shields.io/hexpm/dt/mimic.svg)](https://hex.pm/packages/mimic)\n[![License](https://img.shields.io/hexpm/l/mimic.svg)](https://github.com/edgurgel/mimic/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/edgurgel/mimic.svg)](https://github.com/edgurgel/mimic/commits/master)\n\nA sane way of using mocks in Elixir. It borrows a lot from both Meck & Mox! Thanks [@eproxus](https://twitter.com/eproxus) & [@josevalim](https://twitter.com/josevalim).","ref":"readme.html"},{"type":"extras","title":"Installation - Overview","doc":"Just add `:mimic` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mimic, \"~> 1.7\", only: :test}\n  ]\nend\n```\n\nIf `:applications` key is defined inside your `mix.exs` or you run `mix test --no-start`, you probably want to add `Application.ensure_all_started(:mimic)` in your `test_helper.exs`","ref":"readme.html#installation"},{"type":"extras","title":"Using - Overview","doc":"Modules need to be prepared so that they can be used.\n\nYou must first call `copy` in your `test_helper.exs` for\neach module that may have the behaviour changed.\n\n```elixir\nMimic.copy(Calculator)\n\nExUnit.start()\n```\n\nCalling `copy` will not change the behaviour of the module.\n\nThe user must call `stub/1`, `stub/3`, `expect/4` or `reject/1` so that the functions can\nbehave differently.\n\nThen for the actual tests one could use it like this:\n\n```elixir\nuse ExUnit.Case, async: true\nuse Mimic\n\ntest \"invokes mult once and add twice\" do\n  Calculator\n  |> stub(:add, fn x, y -> :stub end)\n  |> expect(:add, fn x, y -> x + y end)\n  |> expect(:mult, 2, fn x, y -> x * y end)\n\n  assert Calculator.add(2, 3) == 5\n  assert Calculator.mult(2, 3) == 6\n\n  assert Calculator.add(2, 3) == :stub\nend\n```","ref":"readme.html#using"},{"type":"extras","title":"Stub, Expect and Reject - Overview","doc":"","ref":"readme.html#stub-expect-and-reject"},{"type":"extras","title":"Stub - Overview","doc":"`stub/1` will change every module function to throw an exception if called.\n\n```elixir\nstub(Calculator)\n\n** (Mimic.UnexpectedCallError) Stub! Unexpected call to Calculator.add(3, 7) from #PID<0.187.0>\n     code: assert Calculator.add(3, 7) == 10\n```\n\n`stub/3` changes a specific function to behave differently. If the function is not called no verification error will happen.","ref":"readme.html#stub"},{"type":"extras","title":"Expect - Overview","doc":"`expect/4` changes a specific function and it works like a queue of operations. It has precedence over stubs and if not called a verification error will be thrown.\n\nIf the same function is called with `expect/4` the order will be respected:\n\n```elixir\nCalculator\n|> stub(:add, fn _x, _y -> :stub end)\n|> expect(:add, fn _, _ -> :expected_1 end)\n|> expect(:add, fn _, _ -> :expected_2 end)\n\nassert Calculator.add(1, 1) == :expected_1\nassert Calculator.add(1, 1) == :expected_2\nassert Calculator.add(1, 1) == :stub\n```\n\n`expect/4` has an optional parameter which is the amount of calls expected:\n\n```elixir\nCalculator\n|> expect(:add, 2, fn x, y -> {:add, x, y} end)\n\nassert Calculator.add(1, 3) == {:add, 1, 3}\nassert Calculator.add(4, 5) == {:add, 4, 5}\n```\n\nWith `use Mimic`, verification `expect/4` function call of is done automatically on test case end. `verify!/1` can be used in case custom verification timing required:\n\n```elixir\nCalculator\n|> expect(:add, 2, fn x, y -> {:add, x, y} end)\n\n# Will raise error because Calculator.add is not called\n# ** (Mimic.VerificationError) error while verifying mocks for #PID<0.3182.0>:\n#   * expected Calculator.add/2 to be invoked 1 time(s) but it has been called 0 time(s)\nverify!()\n```\n\nUsing `expect/4` on intra-module functions will not work, unless the function is referenced by it's fully qualified name. \n\n```elixir\ndefmodule Calculator do\n  def mult(x, y) do\n    x * y\n  end\n\n  def negation(x) do\n    mult(x, -1)\n  end\nend\n\nCalculator\n|> expect(:mult, fn x, y -> x + y end)\n\nassert Calculator.negation(5) == -5\n\n# Will raise error because because BEAM optimises this case and jumps directly to the appropriate bytecode.\n# ** (Mimic.VerificationError) error while verifying mocks for #PID<0.207.0>:\n#   * expected Calculator.mult/2 to be invoked 1 time(s) but it has been called 0 time(s)\nverify!()\n```\n\nTo ensure that the stubbed Mimic function is called, it can be referenced by `Calculator.mult/2` instead of `mult/2`.","ref":"readme.html#expect"},{"type":"extras","title":"Reject - Overview","doc":"One may want to reject calls to a specific function. `reject/1` can be used to achieved this behaviour.\n\n```elixir\nreject(&Calculator.add/2)\nassert_raise Mimic.UnexpectedCallError, fn -> Calculator.add(4, 2) end\n```","ref":"readme.html#reject"},{"type":"extras","title":"Private and Global mode - Overview","doc":"The default mode is private which means that only the process\nand explicitly allowed process will see the different behaviour.\n\nCalling `allow/2` will permit a different pid to call the stubs and expects from the original process.\n\nIf you are using `Task` there is no need to use global mode as Tasks can see the same expectations and stubs from the calling process.\n\nGlobal mode can be used with `set_mimic_global` like this:\n\n```elixir\nsetup :set_mimic_global\n\ntest \"invokes add and mult\" do\n  Calculator\n  |> expect(:add, fn x, y -> x + y end)\n  |> expect(:mult, fn x, y -> x * y end)\n\n  parent_pid = self()\n\n  spawn_link(fn ->\n    assert Calculator.add(2, 3) == 5\n    assert Calculator.mult(2, 3) == 6\n\n    send parent_pid, :ok\n  end)\n\n  assert_receive :ok\nend\n```\n\nThis means that all processes will get the same behaviour\ndefined with expect & stub. This option is simpler but tests running\nconcurrently will have undefined behaviour. It is important to run with `async: false`.\nOne could use `:set_mimic_from_context` instead of using `:set_mimic_global` or `:set_mimic_private`. It will be private if `async: true`, global otherwise.","ref":"readme.html#private-and-global-mode"},{"type":"extras","title":"DSL Mode - Overview","doc":"To use DSL Mode `use Mimic.DSL` rather than `use Mimic` in your test.  DSL Mode enables a more expressive api to the Mimic functionality.\n\n```elixir\n  use Mimic.DSL\n\n  test \"basic example\" do\n    stub Calculator.add(_x, _y), do: :stub\n    expect Calculator.add(x, y), do: x + y\n    expect Calculator.mult(x, y), do: x * y\n\n    assert Calculator.add(2, 3) == 5\n    assert Calculator.mult(2, 3) == 6\n\n    assert Calculator.add(2, 3) == :stub\n  end\n```","ref":"readme.html#dsl-mode"},{"type":"extras","title":"Stubs with fake module - Overview","doc":"`stub_with/2` enable substitute function call of a module with another similar module\n\n```elixir\n  defmodule BadCalculator do\n    def add(x, y), do: x*y\n    def mult(x, y), do: x+y\n  end\n\n  test \"basic example\" do\n    stub_with(Calculator, BadCalculator)\n\n    assert Calculator.add(2, 3) == 6\n    assert Calculator.mult(2, 3) == 5\n  end\n```","ref":"readme.html#stubs-with-fake-module"},{"type":"extras","title":"Implementation Details & Performance - Overview","doc":"After calling `Mimic.copy(MyModule)`, calls to functions belonging to this module will first go through an ETS table to check which pid sees what (stubs, expects or call original).\n\nIt is really fast but it won't be as fast as calling a no-op function. Here's a very simple benchmark:\n\n```elixir\ndefmodule Enumerator do\n def to_list(x, y), do: Enum.to_list(x..y)\nend\n```\n\nBenchmarking `Enumerator.to_list(1, 100)` :\n\n```\nName               ips        average  deviation         median         99th %\nmimic         116.00 K        8.62 μs   ±729.13%           5 μs          29 μs\noriginal       19.55 K       51.15 μs   ±302.46%          34 μs         264 μs\n\nComparison:\nmimic         116.00 K\noriginal       19.55 K - 5.93x slower\n```\n\nBenchmarking `Enumerator.to_list(1, 250)` :\n\n```\nName               ips        average  deviation         median         99th %\noriginal      131.49 K        7.61 μs   ±167.90%           7 μs          16 μs\nmimic         105.47 K        9.48 μs   ±145.21%           9 μs          27 μs\n\nComparison:\noriginal      131.49 K\nmimic         105.47 K - 1.25x slower\n```\n\nThere's a small fixed price to pay when mimic is used but it is unnoticeable for tests purposes.","ref":"readme.html#implementation-details-performance"},{"type":"extras","title":"Acknowledgements - Overview","doc":"Thanks to [@jimsynz](https://github.com/jimsynz)  and [@alissonsales](http://github.com/alissonsales) for all the help! :tada:\n\nThanks to [@mendokusai](https://github.com/mendokusai) for the nice logo!","ref":"readme.html#acknowledgements"},{"type":"extras","title":"Copyright and License - Overview","doc":"Copyright (c) 2016 Eduardo Gurgel\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}